#define SERIALBUFFERSIZE 256
static uint8_t serialBuffer[SERIALBUFFERSIZE]; // this hold the imcoming string from serial O string
static uint8_t receiverIndex;
static uint8_t dataSize;
static uint8_t cmdMSP;
static uint8_t rcvChecksum;
static uint8_t readIndex;
/**************************************************************************/

void serialMSPreceive()
{
  uint8_t c;
  static enum _serial_state {
    IDLE,
    HEADER_START,
    HEADER_M,
    HEADER_ARROW,
    HEADER_SIZE,
    HEADER_CMD,
  }
  c_state = IDLE;

  while (MSP_PORT.available())
  {
    c = MSP_PORT.read();

    if (c_state == IDLE)
    {
      c_state = (c == '$') ? HEADER_START : IDLE;
    }
    else if (c_state == HEADER_START)
    {
      c_state = (c == 'M') ? HEADER_M : IDLE;
    }
    else if (c_state == HEADER_M)
    {
      c_state = (c == '>') ? HEADER_ARROW : IDLE;
    }
    else if (c_state == HEADER_ARROW)
    {
      if (c > SERIALBUFFERSIZE)
      { // now we are expecting the payload size
        c_state = IDLE;
      } else {
        dataSize = c;
        c_state = HEADER_SIZE;
        rcvChecksum = c;
      }
    } else if (c_state == HEADER_SIZE) {
      c_state = HEADER_CMD;
      cmdMSP = c;
      rcvChecksum ^= c;
      receiverIndex = 0;
    } else if (c_state == HEADER_CMD) {
      rcvChecksum ^= c;
      if (receiverIndex == dataSize) // received checksum byte
      {
        if (rcvChecksum == 0) {
          serialMSPCheck();
        }
        c_state = IDLE;
      } else {
        serialBuffer[receiverIndex++] = c;
      }
    }
  }
}

/********************************************************************/
void serialMSPCheck()
{
  readIndex = 0;
  if (cmdMSP == MSP_LOC)
  {
    LON = read32();
    LAT = read32();
    ALT = read16();
    VARIO = read16();
    ok = true;
//    Debug_PORT.println(">>> MSP_LOC <<<");
  } else if (cmdMSP == MSP_COMP_GPS) {
    distanceToHome = read16();
    directionToHome = read16();
    read8(); //missing
    GPS_time = read32();        //local time of coord calc - haydent
//    Debug_PORT.println(">>> MSP_COMP_GPS <<<");
  }
  else if (cmdMSP == MSP_ALTITUDE) {
    ALT = read32();
    VARIO = read16();
    ok = true;
//    Debug_PORT.println(">>> ALT <<<");
  }
  else if (MSP_RAW_GPS) {
    FIX = read8(); // GPS_fix =
    NUM_SAT = read8(); // GPS_numSat =
    LAT = read32();
    LON = read32();
    GPS_ALT = read16(); // GPS_altitude =
    SPEED =  read16(); // GPS_speed = );
//    Debug_PORT.println(">>> RAW <<<");
    ok = true;
  } else if (cmdMSP == MSP_ANALOG){
    VBAT = read8();
    read16(); // pMeterSum
    RSSI = read16();
//    Debug_PORT.println(">>> MSP_ANALOG <<<");
  }

}
#if defined(Debug_PORT)
void SIMPLEX() {
  Debug_PORT.println(read8());
  Debug_PORT.println(read32());
  Debug_PORT.println(read32());
  Debug_PORT.println(read16());
  Debug_PORT.println(read16());
  Debug_PORT.println("**************************");
}
#endif

/**************************************************************************/
uint8_t read8()  {
  return serialBuffer[readIndex++];
}

uint16_t read16() {
  uint16_t t = read8();
  t |= (uint16_t)read8() << 8;
  return t;
}

uint32_t read32() {
  uint32_t t = read16();
  t |= (uint32_t)read16() << 16;
  return t;
}

/********************************************************************/
